%This script takes the time domain from the observation point data exported
%by OptiFDTD and obtains the spectrum.  It uses the example from 
%https://www.mathworks.com/help/matlab/matlab_prog/local-functions-in-scripts.html
%for the matlab portion.

%Setup Environment
clear('all');
close('all');

%Read in the datafile you saved in OptiFDTD Script
Ey = dlmread('Ey.f2d', '', 2);

%Define Plotting stuff
FrequencyMin = 0;
FrequencyMax = 400e12;

L = length(Ey);%Get num data points
T = Ey(2,1)-Ey(1,1);T=T*1e-15; %Period is step size in time
Fs = 1/T;%Sampling frequency is 1/T
f = Fs*(0:(L/2))/L; %Calculate Frequency

Y = fft(Ey(:,2));%Calculate the FFT


%Get the real and the imaginary.  The cleanup function is used to return a
%single sided FFT where Matlab calculates a double sided one.
Amp = abs(Y/L); Amp = CleanUp(Amp, L);
R = real(Y/L); R = CleanUp(R, L);
I = imag(Y/L); I = CleanUp(I, L);
Phase = atan(I./R); Phase = CleanUp(Phase, L);

plot(f,R, 'b', f, I, 'r', f, Amp, ':k') 
title('Single-Sided Amplitude Spectrum of X(t)')
xlabel('f (Hz)')
ylabel('Respective Magnitude')
xlim([FrequencyMin FrequencyMax]); 

legend('Real', 'Imaginary', 'Amplitude');
figure(2); plot(f, Phase, '-k')
xlim([FrequencyMin FrequencyMax]);

function arr = CleanUp(dft, L)
    arr = dft(1:L/2+1);
    arr(2:end-1) = 2*arr(2:end-1);
end
